
//////////////////////////////////
// Test Avalon Slave
//////////////////////////////////
module test_avalon_slave(
	// slave things
    writedata_hps, readdata_hps, address_hps, write_en_hps, read_hps,
	// other things
	clock, reset, debug
);

// Control IO
input reset;
input clock;
output reg [31:0] debug ;

// Slave IO
input [31:0] writedata_hps ;
input read_hps ;
reg [31:0] writedata_fpga ;
input write_en_hps ;
reg write_en_fpga, write_en_hps_2 ;
output [31:0] readdata_hps ;
wire [31:0] readdata_fpga ;
input [7:0] address_hps ;
reg [7:0] address_fpga ;

// M10K block
true_dual_port_ram_single_clock M10K_block(
	.data_a(writedata_hps), 
	.data_b(writedata_fpga),// Fpga side
	.addr_a(address_hps), 
	.addr_b(address_fpga), // Fpga side
	.we_a(write_en_hps_2 & ~read_hps),
	.we_b(write_en_fpga), // Fpga side
	.clk(clock),
	.q_a(readdata_hps), 
	.q_b(readdata_fpga) // fpga side
);

reg [31:0] count, next_count;
reg [31:0] fpga_count, increment;
reg [2:0] state;

always@(posedge clock) begin
    if(reset) begin 
        count <= 0;
        fpga_count <= 0;
		  increment <= 0;
        state <= 0;
        write_en_hps_2 <= 0;
    end
    else if (state == 0) begin
        
        count <= next_count;
        fpga_count <= fpga_count + increment;
        if(write_en_hps) begin
             write_en_hps_2 <= 1;
             address_fpga <= 6;
             state <= 1;
        end else begin
             write_en_hps_2 <= 0;
             address_fpga <= 3;
        end
    end
    else if (state == 1) begin
        write_en_hps_2 <= 1;
        state <= 2;
    end
    else if (state == 2) begin
        write_en_hps_2 <= 0;
		  address_fpga <= 3;
        state <= 3;
    end
	 else if (state == 3) begin
        write_en_hps_2 <= 1;
		  increment <= readdata_fpga;
        state <= 0;
    end
    else begin
        address_fpga <= 3;
        count <= next_count;
        fpga_count <= fpga_count + increment;
    end
end

always@(*) begin
    write_en_fpga = 0;
    next_count = count + 1;
    //next_fpga_count = fpga_count + readdata_fpga;
    debug = fpga_count;
end



endmodule

//////////////////////////////////
// M10K Blocks
//////////////////////////////////
module true_dual_port_ram_single_clock
#(parameter DATA_WIDTH = 32, ADDR_WIDTH = 8)
(
	input [(DATA_WIDTH-1):0] data_a, data_b,
	input [(ADDR_WIDTH-1):0] addr_a, addr_b,
	input we_a, we_b, clk,
	output reg [(DATA_WIDTH-1):0] q_a, q_b
);


	// Declare the RAM variable
	reg [DATA_WIDTH-1:0] ram[2**ADDR_WIDTH-1:0] /* synthesis ramstyle = "no_rw_check, M10K" */ ; 

	always @ (posedge clk)
	begin // Port a
		if (we_a)
		begin
			ram[addr_a] <= data_a;
			q_a <= data_a;
		end
		else
			q_a <= ram[addr_a];
	end
	always @ (posedge clk)
	begin // Port b
		if (we_b)
		begin
			ram[addr_b] <= data_b;
			q_b <= data_b;
		end
		else
			q_b <= ram[addr_b];
	end
endmodule
